{thread that runs in the background & looks for Bluetooth devices}
unit Scanthread;

interface

uses
  shellapi, Classes, Windows, SysUtils, inifiles, dialogs, forms,stdctrls,
  JwaBluetoothAPIs,utils,systemlock;

type
  TScanThread = class(TThread)
  private
   labelToUpdate: TLabel;
   lbLocalRadios, lbRemoteDevices: TListBox;
   memoToUpdate: TMemo;
   sleepSeconds: integer;
   timeout,lockOnMissedSweeps,sweepsSoFar: integer;
   lockDevice,launchOnLock,launchOnLockParams,launchOnLockDirectory: string;
   launchOnUnlock,launchOnUnlockParams,launchOnUnlockDirectory: string;
   clearLockDeviceFieldOnLock,useWindowsLock: boolean;
   lockedTitle,lockedText,lockedImage,unlockPassword: string;
   labelCaption, itemToAddToLocalListBox, itemToAddToRemoteListBox, itemToAddToMemo: string;
   stringsToAddToRemoteListBox: TStrings;
   frmOwner: TForm;
   procedure updateLabel;
   procedure findBluetoothDevices;
   function BTAdrToStr(const Adr: BLUETOOTH_ADDRESS): string;
   procedure addItemToLocalListBox;
   procedure addItemToRemoteListBox;
   procedure addStringsToRemoteListBox;
   procedure addItemToMemo;
   procedure log(line: string);
  protected
    procedure Execute; override;
  public
    constructor create(_frmOwner: TForm; _labelToUpdate: TLabel; _lbLocalRadios: TListBox;
      _lbRemoteDevices: TListBox; _memoToUpdate: TMemo; _sleepSeconds: integer;
      _timeout: integer; _launchOnLock: string; _launchOnLockParams: string; _launchOnLockDirectory: string;
      _launchOnUnlock: string; _launchOnUnlockParams: string; _launchOnUnlockDirectory: string;
      _clearLockDeviceFieldOnLock: boolean; _lockOnMissedSweeps: integer; _useWindowsLock: boolean;
      _lockedTitle,_lockedText,_lockedImage,_unlockPassword: string);
    destructor Destroy; override;
    procedure setLockDevice(_lockDevice: string);
    function isTerminated: boolean;
    procedure setTimeout(_timeout: integer);
  end;

implementation


uses
   unit1;


{constructor}
constructor TScanThread.create(_frmOwner: TForm; _labelToUpdate: TLabel; _lbLocalRadios: TListBox;
   _lbRemoteDevices: TListBox; _memoToUpdate: TMemo; _sleepSeconds: integer;
   _timeout: integer; _launchOnLock: string; _launchOnLockParams: string; _launchOnLockDirectory: string;
   _launchOnUnlock: string; _launchOnUnlockParams: string; _launchOnUnlockDirectory: string;
   _clearLockDeviceFieldOnLock: boolean; _lockOnMissedSweeps: integer; _useWindowsLock: boolean;
   _lockedTitle,_lockedText,_lockedImage,_unlockPassword: string);
begin
   inherited create(true); // create but don't start running yet

   frmOwner := _frmOwner;
   sleepSeconds := _sleepSeconds;
   timeout := _timeout;
   labelToUpdate := _labelToUpdate;
   lbLocalRadios := _lbLocalRadios;
   lbRemoteDevices := _lbRemoteDevices;
   memoToUpdate := _memoToUpdate;
   lockDevice := '';
   launchOnLock :=_launchOnLock;
   launchOnLockParams := _launchOnLockParams;
   launchOnLockDirectory := _launchOnLockDirectory;
   launchOnUnlock :=_launchOnUnlock;
   launchOnUnlockParams := _launchOnUnlockParams;
   launchOnUnlockDirectory := _launchOnUnlockDirectory;
   clearLockDeviceFieldOnLock := _clearLockDeviceFieldOnLock;
   lockOnMissedSweeps := _lockOnMissedSweeps;
   useWindowsLock := _useWindowsLock;
   lockedTitle := _lockedTitle;
   lockedText := _lockedText;
   lockedImage := _lockedImage;
   unlockPassword := _unlockPassword;
   log('Thread starting');
end;

{destructor}
destructor TScanThread.Destroy;
begin
   inherited destroy;
end;


procedure TScanThread.setLockDevice(_lockDevice: string);
begin
   lockDevice := _lockDevice;
   if (lockDevice = '') then
      log('Lock device cleared')
   else
      log('Lock device changed to ' + lockDevice);
   sweepsSoFar := 0;
end;

procedure TScanThread.updateLabel;
begin
    labelToUpdate.caption := labelCaption;
end;


{main thread entry point}
procedure TScanThread.Execute;
begin
   sweepsSoFar := 0;

   repeat
      labelCaption := 'Discovered Bluetooth devices (refreshing)';
      synchronize(updateLabel);

	   findBluetoothDevices;

      if not terminated then
      begin
         labelCaption := 'Discovered Bluetooth devices';
         synchronize(updateLabel);
         log('Sleeping for ' + IntToStr(sleepSeconds) + ' seconds');
         sleep(sleepSeconds * 1000);
      end;

   until terminated;
   log('Thread terminated');
end;


{returns a string version of a Windows error code - pass in the result of a call to GetLastError}
function getSystemErrorString(lastError: integer): string;
var
  szBuffer: array[0..1024] of char;
begin
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, nil, lastError, 0, szBuffer, sizeof(szBuffer)-1, nil);
  result := StrPas(szBuffer);
end;


procedure TScanThread.findBluetoothDevices;
var
   frmForm1: TForm1;
	hFind: HBLUETOOTH_RADIO_FIND;
	hDevFind: HBLUETOOTH_DEVICE_FIND;
	FindParams: BLUETOOTH_FIND_RADIO_PARAMS;
	SearchParams: BLUETOOTH_DEVICE_SEARCH_PARAMS;
	SearchParamsSize: dword;
	DevInfo: PBLUETOOTH_DEVICE_INFO;
	DevInfoSize: dword;
	hRadio: THandle;
	RadioInfo: PBLUETOOTH_RADIO_INFO;
	RadioInfoSize: dword;
   foundDevices: TStringList;
   fullDeviceName: string;
   lastError, rand: integer;
begin
   frmForm1 := frmOwner as TForm1;
	inc(sweepsSoFar);
   foundDevices := TStringList.create;

	// first we iterate over the local radios
   synchronize(lbLocalRadios.clear);
	FindParams.dwSize := SizeOf(BLUETOOTH_FIND_RADIO_PARAMS);
	hFind := BluetoothFindFirstRadio(@FindParams, hRadio);
   lastError := GetLastError();
	if hFind <> 0 then
	begin
		repeat

			// grab info about this radio
			New(RadioInfo);
			RadioInfoSize := SizeOf(BLUETOOTH_RADIO_INFO);
			FillChar(RadioInfo^, RadioInfoSize, 0);
			RadioInfo^.dwSize := RadioInfoSize;
			BluetoothGetRadioInfo(hRadio, RadioInfo^);
         log('Found local radio ' + RadioInfo^.szName);
         itemToAddToLocalListBox := RadioInfo^.szName;
			synchronize(addItemToLocalListBox);

			// create an empty structure to copy device info into
			New(DevInfo);
			DevInfoSize := SizeOf(BLUETOOTH_DEVICE_INFO);
			FillChar(DevInfo^, DevInfoSize, 0);
			DevInfo^.dwSize := DevInfoSize;

			// we will search for devices found by this radio
         log('Sweep ' + IntToStr(sweepsSoFar) + ', searching for remote devices using local radio ' + RadioInfo^.szName + ', timeout=' + inttostr(timeout) + ' seconds');
			SearchParamsSize := SizeOf(BLUETOOTH_DEVICE_SEARCH_PARAMS);
			FillChar(SearchParams, SearchParamsSize, 0);
			SearchParams.dwSize := SearchParamsSize;
         SearchParams.fReturnAuthenticated := frmForm1.cbReturnAuthenticated.checked;
			SearchParams.fReturnRemembered := frmForm1.cbReturnRemembered.checked;
			SearchParams.fReturnUnknown := frmForm1.cbReturnUnknown.checked;
			SearchParams.fReturnConnected := frmForm1.cbReturnConnected.checked;;
			SearchParams.fIssueInquiry := frmForm1.cbIssueInquiry.checked;
			SearchParams.hRadio := hRadio;   // if null, uses all bluetooth radios
			SearchParams.cTimeoutMultiplier := round(timeout / 1.28) ; // A value that indicates the time out for the inquiry, expressed in increments of 1.28 seconds. For example, an inquiry of 12.8 seconds has a cTimeoutMultiplier value of 10. The maximum value for this member is 45.

         // testing only
         {
         for rand := 1 to 10 do
         begin
            if random(10) > 2 then
            begin
               fullDeviceName := 'Test device ' + inttostr(rand);
               foundDevices.add(fullDeviceName);
               if lbRemoteDevices.Items.IndexOf(fullDeviceName) = -1 then
               begin
                  itemToAddToRemoteListBox := fullDeviceName;
                  synchronize(addItemToRemoteListBox);
               end;
            end;
         end;
         }

			// iterate over devices found by this radio
			hDevFind := BluetoothFindFirstDevice(SearchParams, DevInfo^);
			if hDevFind <> 0 then
			begin
				repeat
               fullDeviceName := string(DevInfo^.szName) + ' - ' + BTAdrToStr(DevInfo^.Address);
               log('Found remote device ' + fullDeviceName + ', last seen ' + datetimetostr(systemtimetodatetime(DevInfo^.stLastSeen)));
               foundDevices.add(fullDeviceName);

               // if this is a new device, display it immediately
               if lbRemoteDevices.Items.IndexOf(fullDeviceName) = -1 then
               begin
                  itemToAddToRemoteListBox := fullDeviceName;
                  synchronize(addItemToRemoteListBox);
               end;

					// erase the dev info just in case
					FillChar(DevInfo^, DevInfoSize, 0);

				until not BluetoothFindNextDevice(hDevFind, DevInfo^);

				if not BluetoothFindDeviceClose(hDevFind) then log('Error closing find device');
			end;

         if foundDevices.Count = 0 then
         begin
            log('Local radio ' + RadioInfo^.szName + ' did not find any remote devices');
         end
         else
         begin
            if foundDevices.IndexOf(lockDevice) > -1 then
            begin
               log('Lock device found, resetting sweep count');
               sweepsSoFar := 0;
               if (not useWindowsLock) and (isLocked) then
               begin
                  if length(launchOnUnlock) > 0 then
                     ShellExecute(self.handle, 'open', pchar(launchOnUnlock), pchar(launchOnUnlockParams), pchar(launchOnUnlockDirectory), SW_SHOWNORMAL);
                  unlockSystem;
               end;
            end
            else if lockDevice <> '' then
            begin
               log('Did not find lock device');
            end;
         end;

			dispose(DevInfo);
         dispose(RadioInfo);

		until not BluetoothFindNextRadio(hFind, hRadio);

		BluetoothFindRadioClose(hFind);
   end
   else
   begin
      log('Could not find any installed Bluetooth radios on this PC, last error = ' + inttostr(lastError));
      log('Error text: ' + getSystemErrorString(lastError));
      log('Visit Blue Lock web site for hints on how to detect your Bluetooth radio.');
      log('Web site is http://members.lycos.co.uk/wuul/bluelock/readme.html');
      terminate;
   end;


   synchronize(lbRemoteDevices.Clear);
   stringsToAddToRemoteListBox := foundDevices;
   synchronize(addStringsToRemoteListBox);
   foundDevices.free;

	// lock the workstation if we haven't seen our device for a number of sweeps
	if (not terminated) and (sweepsSoFar >= lockOnMissedSweeps) and (lockDevice <> '') then
	begin
      log('Did not find device ' + lockDevice + ' after ' + IntTostr(sweepsSoFar) + ' ' + fixPlurals('sweep', sweepsSoFar) + ', locking system');
      sweepsSoFar := 0; // reset

      if (clearLockDeviceFieldOnLock) and (useWindowsLock) then
         synchronize((frmOwner as TForm1).clearLockDevice);

      if length(launchOnLock) > 0 then
         ShellExecute(self.handle, 'open', pchar(launchOnLock), pchar(launchOnLockParams), pchar(launchOnLockDirectory), SW_SHOWNORMAL);

      if useWindowsLock then
   		LockWorkstation
      else
         lockSystemInThread(unlockPassword, lockedTitle, lockedText, lockedImage);
	end;
end;



function TScanThread.BTAdrToStr(const Adr: BLUETOOTH_ADDRESS): string;
var
  Loop: byte;
begin
  Result := IntToHex(Adr.rgBytes[0], 2);
  for Loop := 1 to 5 do
    Result := IntToHex(Adr.rgBytes[Loop], 2) + ':' + Result;
end;


procedure TScanThread.addItemToLocalListBox;
begin
   lbLocalRadios.items.add(itemToAddToLocalListBox);
end;

procedure TScanThread.addItemToRemoteListBox;
begin
   lbRemoteDevices.items.add(itemToAddToRemoteListBox);
end;

procedure TScanThread.addStringsToRemoteListBox;
begin
   lbRemoteDevices.items.addStrings(stringsToAddToRemoteListBox);
end;


procedure TScanThread.addItemToMemo;
begin
   if memoToUpdate.lines.count > 100 then
      memoToUpdate.lines.clear;
   
   memoToUpdate.lines.add(itemToAddToMemo);
end;


procedure TScanThread.log(line: string);
begin
   itemToAddToMemo := TimeToStr(Now) + ' ' + line;
   synchronize(addItemToMemo);
end;

function TScanThread.isTerminated: boolean;
begin
   result := terminated;
end;

procedure TScanThread.setTimeout(_timeout: integer);
begin
   timeout := _timeout;
   sweepsSoFar := 0;
end;


end.
