unit Unit1;

interface

uses
  shellapi, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, comctrls, inifiles, Buttons, ExtCtrls,scanthread,utils;

const
  WM_NOTIFYICON  = WM_USER+333;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    Panel2: TPanel;
    Label1: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    lbRemoteDevices: TListBox;
    lbLocalRadios: TListBox;
    txtLockDevice: TEdit;
    Button1: TButton;
    Memo1: TMemo;
    Label2: TLabel;
    TrackBar1: TTrackBar;
    Label4: TLabel;
    cbReturnRemembered: TCheckBox;
    cbReturnAuthenticated: TCheckBox;
    cbReturnConnected: TCheckBox;
    cbReturnUnknown: TCheckBox;
    cbIssueInquiry: TCheckBox;
    procedure lbRemoteDevicesClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure txtLockDeviceChange(Sender: TObject);
    procedure clearLockDevice;
    procedure TrackBar1Change(Sender: TObject);

  private
    { Private declarations }
    ScanThread: TScanThread;
  public
    { Public declarations }
    TrayIcon: TNotifyIconData;
    HMainIcon: HICON;
    procedure ClickTrayIcon(var msg: TMessage); message WM_NOTIFYICON;
    Procedure MinimizeClick(Sender:TObject);
  end;

const
   confFile = 'bluelock.conf';
   appVersion = '1.92';

var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.txtLockDeviceChange(Sender: TObject);
begin
   ScanThread.setLockDevice(txtLockDevice.text)
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
   clearLockDevice;
end;

procedure TForm1.lbRemoteDevicesClick(Sender: TObject);
var
   device: string;
begin
   if lbRemoteDevices.itemIndex > -1 then
   begin
   	// copy selected device to lock device textbox
      device := lbRemoteDevices.items[lbRemoteDevices.itemIndex];
      txtLockDevice.text := device;
   end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
	IniFile: TIniFile ;
begin
	IniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + '\' + confFile);
	try
		with IniFile do
		begin
		   WriteString('main', 'lockdevice', txtLockDevice.text);
         WriteString('main', 'timeout', inttostr(trackbar1.position));
         WriteBool('main', 'IssueInquiry', cbIssueInquiry.checked);
         WriteBool('main', 'ReturnRemembered', cbReturnRemembered.checked);
         WriteBool('main', 'ReturnUnknown', cbReturnUnknown.checked);
         WriteBool('main', 'ReturnConnected', cbReturnConnected.checked);
         WriteBool('main', 'ReturnAuthenticated', cbReturnAuthenticated.checked);
		end;
	finally
		IniFile.Free ;
	end ;

   ScanThread.terminate;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
	IniFile: TIniFile ;
   sleepSeconds: integer;
   timeout,lockOnMissedSweeps: integer;
   lockDevice, launchOnLock, launchOnLockParams, launchOnLockDirectory: string;
   launchOnUnlock, launchOnUnlockParams, launchOnUnlockDirectory: string;
   clearLockDeviceFieldOnLock,useWindowsLock: boolean;
   lockedTitle,lockedText,lockedImage,unlockPassword: string;
begin
  self.caption := self.caption + ' ' + appVersion;

  // create system tray icon
  HMainIcon:=LoadIcon(MainInstance, 'MAINICON');
  Shell_NotifyIcon(NIM_DELETE, @TrayIcon);
  with trayIcon do
  begin
    cbSize              := sizeof(TNotifyIconData);
    Wnd                 := handle;
    uID                 := 123;
    uFlags              := NIF_MESSAGE or NIF_ICON or NIF_TIP;
    uCallbackMessage    := WM_NOTIFYICON;
    hIcon               := HMainIcon;
    szTip               := 'Blue Lock';
  end;
  Application.OnMinimize:= MinimizeClick;

   // read config
	IniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + '\' + confFile);
	try
	   with IniFile do
	   begin
			lockDevice := ReadString('main', 'lockdevice', '');
			timeout := StrToInt(ReadString('main', 'timeout', '20'));
         cbIssueInquiry.checked := ReadBool('main', 'IssueInquiry', true);
         cbReturnRemembered.checked := ReadBool('main', 'ReturnRemembered', true);
         cbReturnUnknown.checked := ReadBool('main', 'ReturnUnknown', true);
         cbReturnConnected.checked := ReadBool('main', 'ReturnConnected', true);
         cbReturnAuthenticated.checked := ReadBool('main', 'ReturnAuthenticated', true);
         if timeout > 57 then timeout := 57;
         if timeout < 1 then timeout := 1;
         trackbar1.position := timeout;
         sleepSeconds := ReadInteger('main', 'sleepseconds', 10);
         launchOnLock := ReadString('main', 'launchOnLock', '');
         launchOnLockParams := ReadString('main', 'launchOnLockParams', '');
         launchOnLockDirectory := ReadString('main', 'launchOnLockDirectory', '');
         launchOnUnlock := ReadString('main', 'launchOnUnlock', '');
         launchOnUnlockParams := ReadString('main', 'launchOnUnlockParams', '');
         launchOnUnlockDirectory := ReadString('main', 'launchOnUnlockDirectory', '');
         clearLockDeviceFieldOnLock := ReadString('main', 'clearLockDeviceFieldOnLock', 'true') = 'true';
         lockOnMissedSweeps := ReadInteger('main', 'lockOnMissedSweeps', 1);
         useWindowsLock := ReadString('main', 'useWindowsLock', 'true') = 'true';
         if (not useWindowsLock) and (not hasAdminRights) then
         begin
            useWindowsLock := true;
            ShowMessage('You do not have full admin rights so you cannot use the custom lock feature - reverting to standard Windows lock');
         end;
         lockedTitle := ReadString('main', 'lockedTitle', '');
         lockedText := ReadString('main', 'lockedText', '');
         lockedImage := ReadString('main', 'lockedImage', '');
         unlockPassword := ReadString('main', 'unlockPassword', '');

         if readString('main','checkForNewVersion','true') = 'true' then
         begin
            try
               if checkForNewVersion('http://members.lycos.co.uk/wuul/bluelock/bluelock.xml', StrToFloat(appVersion)) then
               begin
                  if MessageDlg('A newer version of this program is available.  Do you want to visit the web site now so you can download the most recent version?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
                     ShellExecute(GetDesktopWindow, 'open', PChar('http://members.lycos.co.uk/wuul/bluelock/readme.html'), nil, nil, SW_ShowNormal)
               end;
            except
               ShowMessage('Unable to check for new version.' + chr(13)+chr(10) + chr(13)+chr(10) + 'The site may be down, or there could be a problem with your internet connection.');
            end;
         end;
	   end;
	finally
		IniFile.Free ;
	end ;

   // start main thread
   ScanThread := TScanThread.create(self, label1, lbLocalRadios, lbRemoteDevices, memo1, sleepSeconds,
      timeout, launchOnLock, launchOnLockParams, launchOnLockDirectory, launchOnUnlock,
      launchOnUnlockParams, launchOnUnlockDirectory, clearLockDeviceFieldOnLock, lockOnMissedSweeps, useWindowsLock,
      lockedTitle,lockedText,lockedImage,unlockPassword);
   txtLockDevice.text := lockDevice;
   ScanThread.FreeOnTerminate := True ;
   ScanThread.resume;

   // only for screenshot - fake some items
   // as I'm way too paranoid to publish a screenshot showing my device...hehe
   {lbLocalRadios.items.add('Cantrell_laptop');
   lbRemoteDevices.items.add('Kia_PDA - 01:0D:DE:9C:B4:6F, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('SonyVaio - 03:5D:DF:9D:29:1A, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('Cantrell_Nokia - 42:64:6B:7A:18:DD, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('Nokia 6610 - 17:5D:DF:9D:29:1A, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('Randy''s laptop - 27:33:9A:1F:22:1B, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('AVI_Cell - 18:92:3A:3B:22:82, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('Beryl_T610 - 91:12:3B:21:8E:E2, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.items.add('H_Kepler - 18:13:61:76:2E:EF, last seen ' + DateTimeToStr(now));
   lbRemoteDevices.ItemIndex := 2;
   txtLockDevice.text := 'Cantrell_Nokia - 42:64:6B:7A:18:DD';}
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
   Shell_NotifyIcon(NIM_Delete, @TrayIcon);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
   ShellExecute(GetDesktopWindow, 'open', PChar('http://members.lycos.co.uk/wuul/bluelock/readme.html'), nil, nil, SW_ShowNormal);
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
   label4.Caption := 'Timeout (' + inttostr(TrackBar1.position) + ' seconds)';
   if ScanThread <> nil then
      ScanThread.setTimeout(TrackBar1.position);
end;

Procedure TForm1.MinimizeClick(Sender:TObject);
begin
   Shell_NotifyIcon(NIM_Add, @TrayIcon);
   hide;
   {hide the taskbar button}
   if IsWindowVisible(Application.Handle)
   then ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TForm1.ClickTrayIcon(var msg: TMessage);
begin
  case msg.lparam of
    WM_LBUTTONUP, WM_LBUTTONDBLCLK :
    {WM_BUTTONDOWN may cause next Icon to activate if this icon is deleted -
        (Icons shift left and left neighbor will be under mouse at ButtonUp time)}
    begin
      Application.Restore;  {restore the application}
      If WindowState = wsMinimized then WindowState := wsNormal;  {Reset minimized state}
      {Added 5/6/04 ====>} visible:=true;
      SetForegroundWindow(Application.Handle); {Force form to the foreground }
      Shell_NotifyIcon(NIM_Delete, @TrayIcon);
    end;
  end;
end;


procedure TForm1.clearLockDevice;
begin
   txtLockDevice.text := '';
end;



end.


